@@ -402,7 +402,7 @@ my %riffEncoding = ( #2
 );
 
 # add our composite tags
-Image::ExifTool::AddCompositeTags(\%Image::ExifTool::RIFF::Composite);
+Image::ExifTool::AddCompositeTags('Image::ExifTool::RIFF');
 
 
 #------------------------------------------------------------------------------
@@ -588,7 +588,7 @@ including Windows WAV audio and AVI video files.
 
 =head1 AUTHOR
 
-Copyright 2003-2006, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2007, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
